;  (c:ism) Interval Average SMoothing
;  Running averages for all points in an interval
;  1. Points in the range are given equal weight in the average 
;  2. Points are weighted toward the center point in proportion to their
;     distance from the center
;  3. Points are weighted toward the center point in proportion to the
;     log of their distance from the center

(defun c:ism () ; Moving Average Smoothing
(setq old (do_err nil '("Bugs or Error")))
  (foreach i '(apts bpts cpts dpts) (set i nil))
  (prompt "\nSelect graph for Moving Interval Average Smoothing")
(if (setq ent (f_fetchgr))
  (progn
  (if v_ism nil (setq v_ism 3))
  (if v_pin nil (setq v_pin 10))
  (if v_itsi nil (setq v_itsi 1))
  (prompt "\n1=Unweigted Avg., 2=Linear Center Weighted, 3=Log Center Weighted")
  (setq ism (getint (strcat "\nSelect Averaging Function <" (itoa v_ism)">: ")))
   (if ism (setq v_ism ism))
   (setq xdat (strcat "Ism-" (etos v_ism)))
   (setq pin (getreal (strcat"\nEnter interval <"(rtos v_pin)">: ")))
   (if pin (setq v_pin pin))
   (setq its (getint (strcat"\nEnter Number Iterations <"(itoa v_itsi)">: ")))
   (if its (setq v_itsi its))(setq its v_itsi)
   (setq xdat (strcat xdat " " (rtos v_pin) "x" (etos v_itsi)))
   (prompt "....processing....")
   ;;; collect points in coordinates relative to reference point
   (f_prgraf ent 
     '(setq dpts (append dpts (list (f_torel _pt2))) )
     )
(while (> its 0)
    (princ ".")        
    (setq dpts (f_ismpts v_ism v_pin dpts))
;skip, no need if values not recursively valued
; (f_nilptn pn)  ;;;clear variable list needed, and average in backward direction
;    (setq bpts (reverse (f_ismpts v_ism v_pin (reverse dpts))))
;    (setq apts (f_ismpts v_ism v_pin dpts) dpts nil)
;    (mapcar     ;;;average foreward and back runs
;        '(lambda (a b)(setq dpts (append dpts 
;            (list(list (car a)(/ (+ (cadr a)(cadr b))2)))
;            )))
;         apts bpts)
    (setq its (1- its) )
    )
(command ".undo" "g")
(command "pline" )(foreach pt dpts (command (f_toabs pt)))(command "")
(if (or (null v_xdat)(= "" v_xdat))(setq sp "")(setq sp ", "))
(setdata "graph" (f_lastent) (strcat v_xdat sp xdat))
 ))
(command ".undo" "e")
(undo_err old)
(if wc_beep (wc_beep 1000.0 10000.0))
(princ));defun
;----------------------------------------------------------------------------
(defun f_ismpts (alt pin qpts / )
   (setq pts qpts len (length pts) hpin (/ pin 2)
         pno 0 npts nil twt 0
         fflag 1 flag nil wts nil wwts nil)
;for each point
 (while  qpts 
    (setq ipt (car qpts)      ; new subject point
          qpts (cdr qpts)     ; remainder list 
          pno (1+ pno))       ; current point number in full point list
          
    (setq npt nil n 0 fac 0 tot 0 valx 0 valy 0 wpts nil)
    (if (< (caar pts) (- (car ipt) pin ))
        (setq flag 1))
    ;;;Get list of points within the interval
    (setq vpin (min 
            (abs (- (caar pts)(car ipt)))
            (abs (- (car (last pts))(car ipt)))
            hpin))
    (foreach i pts 
        (if (and 
              (<= (- (car ipt) vpin)(car i)) 
              (<= (car i) (+ (car ipt) vpin )) 
             )
            (setq wpts (append wpts (list i)))
            )
          )
;  (pr_ '(wpts))
    ;;;accumumulate total for available points weighted around a center pt
    (foreach i wpts
      (cond ;assign point weight by function choice
          ((= v_ism 1) (setq wt 1 ))
          ;inverse ratio of distance from center
          ((= v_ism 2) (setq wt (- 1 (/ (abs (- (car i)(car ipt))) hpin)) ) 
            )
          ((= v_ism 3) (setq wt (- 1 (/ (abs (- (car i)(car ipt))) hpin))  
                             ;wt (expt (/ wt (+  pin) 1.0) 0.5 )
                             wt (log (+ 1.1 wt))
                          )
                )
          (t nil) 
       ); cond
         (setq valy (cadr i)
                fac (+ fac wt) ;factor total
                tot (+ tot (* wt valy))
                  n (1+ n)
              )
       (if (and flag fflag)
           (setq wts (append wts  (list wt)) ))
      );foreach
;       (if (and flag fflag)
;          (progn 
;             (princ "\nWeights: "))
;;            (pr_ '(wt wts fac twt))
;            (foreach i wts 
;                (princ (strcat " " (rtos (setq w(/ i fac 1.0)))))
;                (setq twt (+ twt w)
;                      WWts (append wwts (list (* 1.0 w))) )
;                      ;;; ..scaled for easy input in 1X figure
;                )
;            (princ (strcat " Total=" (rtos twt)))
;           (pr_ '(ipt n))
;            (setq wts nil twt 0)
;            (setq fflag  nil)
;           ))
   (if (/= FAC 0) (progn
     (setq  y  (/ tot fac))     ;weighted average of available points
     (setq npt (list (car ipt) y))
     ))
    (if npt (setq npts (append npts (list npt)) ))
   );while
npts);defun
