;(C:DIN) (F_DIN)
; Uses subfunctions--- GETDATA, SETDATA, ETOS 
; (F_DXYVAR FLAG) (F_NILPTN V) (F_LISTVARS L br XPT)
; (F_TOABS XPT) (F_TOREL XPT) 
; (F_3SYM DD0 DD2 DDX1 DDY1 DXI DDPT) (F_FETCHGR) 
; (F_PRGRAF SENT SFUN)  (F_DOGRAF XDAT SENT AFUN SFUN DONE) 
; (F_FILGR) (F_GRRFIL) (F_GETPTS)
; Also use, LKX, XTX for Xdata listing & input, Batch for multiple tasking
;
;                   ***********
;       Copyright (c) 1995 Philip F. Henshaw
;
;                     **** graphing programs ****
;  c:din     Inserts new points by interpolation by continuity rules, 
;            1. 3rd deriv offset
;            2. eqaual 3rd deriv infront and behind, original points kept_
;            3. eqaual 3rd deriv infront and behind, and adjusts original points kept_
;  f_fetchgr picking a graph object and gets V_xdat
;  f_dograf xdat sent afun sfun done - processes a graph and creates another according to the function 
;            input and adds xdata recording the history of manipulation
;  f_dxyvar flag - sets dxn dyn variables
;  f_nilptn v - sets v of xn yn variables nil
;  f_listvars l br xpt - revises a br list of xn yn vars, adding pt to end at 
;           list level l
;  f_toabs xpt - returns point in absolute coords  
;  f_torel xpt - returns point in graph _pt0 coords
;
;  f_3sym    calculate point w/ 3rd deriv symmetry within bracket of 5 points
;----------------------------------------------------------------------------
;A the continuity of a time series data set can be approximated by inserting
;  new points according to rules about the maxima and minima of the rates of
;  change posible.
;  1. intermeditate points placed between given points, above or below the
;  average of the two according to sign of 2nd derivative, in proportion to 
;  the local 3rd derivative rate of change.  (3rd deriv factor)(f_din 1)
;  2. intermeditate points placed between given points, above or below the
;  average of the two according to sign of 2nd derivative, to make the 
;  local 3rd derivatives on each side equal (3rd deriv symmetry) (f_din 2)
;  3. intermeditate points placed between given points, above or below the
;  average of the two according to sign of 2nd derivative, to make the 
;  local 3rd derivatives on each side equal, and then adjusting the bracking 
;  original points in the same way. (3rd deriv symmetry w/ smoothing) (f_din 3)
;  4. Calculating intermeditate points to make the 3rd derivative on each 
;  side equal, but then only adjusting the value of the origninal points as 
;  above. (3rd deriv symmetry w/ smoothing) (f_din 4)
(defun c:din () ;Derivative Interpolation
(prompt "\nSelect graph for Derivative Interpolation")
    ;;;select graph object & set value of local origin _pt0
(if (setq ent (f_fetchgr))(progn
  (if v_din nil (setq v_din 2))
  (prompt "\n1=3rd Deriv factor, 2=3rd Deriv Sym, 3=3D Sym w/smoothing, 4=Smooth only")
  (setq din (getint (strcat "Select Interpolation Function <" (itoa v_din)">: ")))
    (if din (setq v_din din))
  (prompt "....processing....")
  (setq xda (strcat "Din" (etos v_din)))
  (f_din)
  ))
(if wc_beep (wc_beep 1000.0 10000.0))
);defun
;----------------------------------------------------------------------------
(defun f_din () ;Trend Interpolation
(setq br 4  n 0 np 0 f1 nil)
   ;;;clear variable list needed
   (f_nilptn br)
   (foreach i '(ptl ptp pto ptn ptm _pt)(set i nil))
   (setq fun '(progn
       ;;;clear values of working variables
     (foreach i '(d0 d1 d2 ptn _pt mptf belt delt felt vdir rev1 rev2 sdir)
           (set i nil))
       ;;; set current point '_pt2' as x0, y0 and moving predeeding three points
           ; for point list xn,yn n = 0,1,2,3 
     (f_listvars 0 br (f_torel _pt2) )
     (setq n  (f_dxyvar 1); set difference variables dx0,dx1,dx2,dy0,dy1,dy2..d0,d1,d2
           np (1+ np))    ;point counter
      ;;;find higher derivatives, signs and reversals
     (if (and d0 d1 ) ;if enough points to have defined d0 and d1
         (setq 2d0 (/ (- d1 d0) (+ dx0 dx1)))) ;2nd deriv, @ 3rd of 4 pts
     (if (and d0 d1 d2)(progn
         (setq 2d1 (/ (- d2 d1) (+ dx1 dx2))  ;2nd deriv, @ 4th of 4 pts
               3d1 (/ (- 2d1 2d0)  dx1 )      ;3rd deriv, @ mid of 4 pts
               3ds (if (zerop 3d1) 1 (/ 3d1 (abs 3d1)))               ;sign of 3d1
               2d0s (if (= d2 d0) 1 (/ (- d2 d0) (abs (- d2 d0))))    ;sign of 3d1
               d1s (if (zerop d1) 1 (/ d1 (abs d1)))                  ;sign of d2
               rev1 (minusp (* d0 d1))      ;T if reversal @ 2nd pt
               rev2 (minusp (* d1 d2))      ;T if reversal @ 3rd pt
               )
      ;;;find set new intermediate point by interpolating local derivative
     (if (member v_din '(2 3 4))
         (progn
            (setq delt 
               (f_3sym d0 d2 dx1 dy1 (/ dx1 2)              
                        (list (/ (+ x2 x1)2)(/ (+ y2 y1)2)) ; location for overshoot mark
              ))
          ))
      ;;; then make adjustments for each option
     (cond 
             ((= v_din 1) ;;; 3rd deriv offset
             (setq      ;fraction of mid-pt ht to be added or subtracted
                mpf  (expt 1.0001 (* 0.01 (abs 3d1)))
                      ;sign correction for offsets
                mptf (/ (- (+ 1  (/ 2d0s  mpf )) 2d0s)4)
                delt  (+ (/ (- y2 y1) 2)(* -1 2d0s (abs dy1) mptf))
               ))
             ((= v_din 2) ; 3rd deriv symmetry
               nil )
             ((= v_din 3) ; 3rd deriv symmetry w/ smoothing
               (setq ptn  (list (/ (+ x1 x2) 2) (+ y1  delt ))
                     po   (f_torel pto) 
                     pl   (f_torel ptl)
                     db   (/(-(cadr pl)y0)(-(car pl)x0))
                     df   (/(- y2 (cadr ptn))(- x2 (car ptn)))
                     dxo  (- (car ptn)(car pl))  ;;;(/(+ dx0 dx1)2)
                     dyo  (- (cadr ptn)(cadr pl))
                     belt (f_3sym db df dxo dyo (- (car po)(car pl)) po)
                     pto  (list (car po) (+ (cadr pl) belt))
                     pto  (f_toabs pto)
                     ptl  (f_toabs pl)
                  )
                )
             ((= v_din 4) ; 3rd deriv smoothing only
               (setq ptn  (list (/ (+ x1 x2) 2) (+ y1  delt ))
                     po   (f_torel pto) 
                     pl   (f_torel ptl)
                     db   (/(-(cadr pl)y0)(-(car pl)x0))
                     df   (/(- y2 (cadr ptn))(- x2 (car ptn)))
                     dxo  (- (car ptn)(car pl))  ;;;(/(+ dx0 dx1)2)
                     dyo  (- (cadr ptn)(cadr pl))
                     belt (f_3sym db df dxo dyo(- (car po)(car pl)) po)
                     pto  (list (car po) (+ (cadr pl) belt))
                     pto  (f_toabs pto)
                     ptl  (f_toabs pl)
                  )
                )
              (t nil))
        )); end analysis of full point lists
      ;;;set and plot adjusted points
     (if (and (= d0 d1)(/= d1 d2)(= 0 d0))(setq delt 0))
     (if (and (/= d0 d1)(= d1 d2)(= 0 d2))(setq delt 0))
         (cond      
           ((and d0 d1 d2)      ;normal - 4 pts defined
               (setq ptn (list (/ (+ x1 x2) 2) (+ y1  delt )) )
               (if (and (member v_din '(2 3 4))(= np 4))
                   ;first, adjust first inserted point 
                   (setq ptl (f_torel ptl)
                          dp (* d0 (/ d0 d1))
                          di (/ (- (cadr ptn) y1) (/ dx1 2))
                         del (f_3sym dp di dx0 dy0 (/ dx0 2) ptl)
                         ptl (f_toabs (list (car ptl) (+ y0 del))) )
                   )
               (setq ptn (f_toabs ptn))
               (if (= 4 v_din)
                 (if (and belt pto) (command  "u" "u" pto ptm))
                 (if (and belt pto) 
                     (if (= np 4) 
                         (command "u" "u" "u" ptl pto ptn ptm)
                         (command "u" "u" pto ptn ptm) )
                     (if (= np 4)
                         (command "u" "u" "u" ptl pto ptn ptm)
                         (command "u" ptn ptm)) )
                )
              (setq _pt (f_toabs (list x3 y3)))
              )
           ((and d0 d1)  ;for only 3 points use temporary pt
               (setq ptn (f_toabs (list (/ (+ x0 x1)2) (/(+ y0 y1)2)))
                     _pt  (f_toabs (list x2 y2)))
               (command "u" ptn ptm)
               )
           ((and d0 )  ;for only 2 points 
               (setq _pt (f_toabs (list x1 y1)))
               )
           (t (setq _pt (f_toabs (list x0 y0))) )
           )
 ;(pr_ '(ptl pto ptn ptm _pt))
; (grread)
      ;;;reset variables for next round for smoothing
          (setq ptp pto pto ptm ptm _pt)
          (setq ptl ptn)
   ));end 'fun
      ;;;plot midpoint of last two points to end the routine
   (setq done '(progn
         (setq ptn (list (/(+ x2 x3)2)(/ (+ y2 y3)2)))
         (cond
           ((= v_din 1) ; 1st deriv symmetry
             (command "u" (f_toabs ptn) _pt )
             )
           ((= v_din 2) ; 3rd deriv symmetry, using foreward projected pt & test
             (if (= d1 0)(setq dd2 d2)(setq dd2 (- (+ d2 d2) d1)  ))
;             (if (= d1 0)(setq dd2 d2)(setq dd2 (* d2 (/ d2 d1))))
             (setq yn (+ y2 (f_3sym d1 dd2 dx2 dy2 (/(+ 0 dx2) 2) ptn)))
;            (setq  yn (- yn  (- yn (* (/(+ y2 y3) 2) 0.5) )) )
;             (setq yn (+ y2 (f_3sym d1 dd2 dx2 dy2 (/ dx2 2) ptn)))
             (setq ptn (list (car ptn) yn))
             (command "u" (f_toabs ptn) _pt )
             )
           ((= v_din 3) ; 3rd deriv symmetry w/ smoothing
             (setq po   (f_torel pto) 
                   pl   (f_torel ptl)
                   db   (/(-(cadr pl)y1)(-(car pl)x1))
                   df   (/(- y3 (cadr ptn))(- x3 (car ptn)))
                   dxo  (- (car ptn)(car pl)) 
                   dyo  (- (cadr ptn)(cadr pl))
                   belt (f_3sym db df dxo dyo(- (car po)(car pl))po)
                   pto  (list (car po) (+ (cadr pl) belt))
                   pto  (f_toabs pto)
                   ptn  (f_toabs ptn)
                )
              (command "u" "u" pto ptn _pt)
              )
           (t nil))
         ))
;         (pr_ '(xda))
   (f_dograf xda ent nil fun done)
);defun
;----------------------------------------------------------------------------
(princ)