; (C:PVIEW) draws a window box around each view in a drawing and 
;* displays its name in the layer status box. It prompts for Next view,
;* Exit, or Restore to the currently indicated view.
;* Requires DXF and user interface functions to be loaded.
;*
;*  AutoLISP Files for Maximizing AutoCAD Volume II - IL DISK Version 11.00
;*   (c) 1987, 1988, 1989, 1991 New Riders Publishing. All Rights Reserved.
;*       Developed by Rustin Gesner, Patrick Haessly and Joseph Smith
;*
;*   Modifications P. Henshaw 1993, 1994
;*
(if (not (and dxf ukword))                              ;test subroutines   
  (prompt "\nRequires DXF and UKWORD functions. Load aborted. ")                    
  (progn                                                ;else load OK

(defun C:PVIEW ( / flag vport tmode tbdata vname Y X CP P1 P2 P3 P4 V)
  (setq $screen (getvar "SCREENSIZE")                ;screen pixel resolution
        aratio (/ (cadr $screen) (car $screen))      ;aspect ratio based on pixels
        tmode (getvar "TILEMODE")
        n 0
        vport (/= 1 (length (vports)))     ;determine if active pspace vports
  )
  (setq old (do_err nil '((command ".ucs" "p"))))
  (command ".ucs" "")
  (while (and (/= V "Exit")(/= V "Restore")(setq tbdata (tblnext "VIEW" T)))
    (while tbdata                                    ;loop through view table
      (setq flag T n (1+ n)                          ;reset
        vname (dxf 2 tbdata)                         ;extracts view name
        Y (* (dxf 40 tbdata) 0.5)                    ;view height
        X (* (dxf 41 tbdata) 0.5)                    ;view width
      );setq
      (if tmode                                      ;R11 code
        (if (= tmode 0)                              ;TILEMODE off
          (if (= 1 (rem (dxf 70 tbdata) 2))          ;if view in pspace
            (if (/= 1 (getvar "CVPORT")) (command "pspace"))                       ;then
            (if vport                                ;else in mspace
              (if (= 1 (getvar "CVPORT")) (command "mspace"))
              (setq tbdata (tblnext "VIEW")          ;get next and exit loop
                flag (prompt (strcat "\nNo active viewports. Mspace view " vname " ignored. "))
              )
          ) );if
          (if (= 1 (rem (dxf 70 tbdata) 2))          ;view in pspace
            (setq tbdata (tblnext "VIEW")            ;get next and exit loop
              flag (prompt (strcat "\nPspace view " vname " ignored. "))
          ) )
      ) );if,if&if
      (if flag                          
        (progn
          ;(if (> X (* Y (/ 1 aratio)))               ;corrects for proportions of 
          ;  (setq Y (* X aratio))                    ;the screen. Either X or Y
          ;  (setq X (* Y (/ 1.0 aratio)))            ;will determine window maximum.
          ;);if
          (setq CP (dxf 10 tbdata)                   ;center point of view
            P1 (list (- (car CP) X) (- (cadr CP) Y)) ;lower left corner
            P2 (list (- (car CP) X) (+ (cadr CP) Y)) ;upper left
            P3 (list (+ (car CP) X) (+ (cadr CP) Y)) ;upper right
            P4 (list (+ (car CP) X) (- (cadr CP) Y)) ;lower right
          );setq
          (grdraw P1 P2 7 1)                         ;draws left vertical vector 
          (grdraw P2 P3 7 1)                         ;draws top horizontal
          (grdraw P3 P4 7 1)                         ;draws right vertical
          (grdraw P4 P1 7 1)                         ;draws bottom horizontal
          (grtext -1 (strcat "View "(itoa n)": " vname))   ;prints name in layer status box
          (setq V (ukword 1 "Next Restore Exit" "Display another VIEW? Exit/Restore/Next/" "Next"))
          (cond
            ((= "Next" V) (setq tbdata (tblnext "VIEW"))) ;gets next view from table
            ((= "Restore" V)                              ;2nd cond, execute VIEW
              (progn
                (command "VIEW" "R" vname)           ;change view
                (setq tbdata nil)            ;nils variable causing while loop termination
            ) );progn and 2nd cond
            ((= "Exit" V) (setq tbdata nil)) ;nils variable causing while loop termination
          );cond
          (grdraw P1 P2 0)   ; each of these draws
          (grdraw P2 P3 0)   ; a black vector over
          (grdraw P3 P4 0)   ; the current screen
          (grdraw P4 P1 0)   ; vectors. It's the only way short of a full redraw
      ) )if&progn
  ) 
  (setq n 0)(PRINC "\nRestarting View Sequence")
  );while and & while tbdata
  (if V 
    (grtext)               ;clears out the temporary screen labels
    (prompt "\nNo views defined. ")
  );if
  (command ".ucs" "p")
  (undo_err old)
  (princ)
);defun C:PVIEW
;*
));test subroutines
