;(ssx)(c:ssx) property selection
;;;   ssx.lsp
;;;   Copyright (C) 1990 by Autodesk, Inc.
;;;             (C) 1995 HDS to add SCale and Multi-level CAtch
;;;  
;;;   Permission to use, copy, modify, and distribute this software and its
;;;   documentation for any purpose and without fee is hereby granted.  
;;;
;;;   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY. 
;;;   ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF 
;;;   MERCHANTABILITY ARE HEREBY DISCLAIMED.
;;; 
;;;   Larry Knott                        Version 2.0    7/18/88  
;;;   Carl Bethea & Jan S. Yoder         Version 3.0
;;;   Enhancements to (ssx).
;;;   15 March 1990  
;;;
;;;--------------------------------------------------------------------------;
;;; DESCRIPTION
;;;                              SSX.LSP                                    
;;;                                                                          
;;;   "(SSX)" -  Easy SSGET filter routine.                                    
;;;                                                                            
;;;   Creates a selection set.  Either type "SSX" at the "Command:" prompt   
;;;   to create a "previous" selection set or type "(SSX)" in response to 
;;;   any "Select objects:" prompt.  You may use the functions "(A)" to add 
;;;   entities and "(R)" to remove entities from a selection set during 
;;;   object selection.  More than one filter criteria can be used at a 
;;;   time.         
;;;                                                                          
;;;   SSX returns a selection set either exactly like a selected
;;;   entity or, by adjusting the filter list, similar to it.
;;;
;;;   The initial prompt is this:
;;;   
;;;     Command: ssx
;;;     Select object/<None>: (RETURN)
;;;     >>Block name/Color/Entity/Flag/LAyer/LType/Pick/Style/Thickness/Ucs: 
;;;   
;;;   Pressing RETURN at the initial prompt gives you a null selection 
;;;   mechanism just as (ssx) did in Release 10, but you may select an 
;;;   entity if you desire.  If you do so, then the list of valid types 
;;;   allowed by (ssget "x") are presented on the command line.
;;;   
;;;     Select object/<None>:  (a LINE selected)
;;;     Filter: ((0 . "LINE") (8 . "0") (39 . 2.0) (62 . 1) (210 0.0 0.0 1.0)) 
;;;     >>Block name/Color/Entity/Flag/LAyer/LType/Pick/Style/Thickness/Ucs: 
;;;   
;;;   At this point any of these filters may be removed by selecting the 
;;;   option keyword, then pressing RETURN.
;;;   
;;;     >>Layer name to add/<RETURN to remove>: (RETURN)
;;;         
;;;     Filter: ((0 . "LINE") (39 . 2.0) (62 . 1) (210 0.0 0.0 1.0))
;;;     >>Block name/Color/Entity/Flag/LAyer/LType/Pick/Style/Thickness/Ucs: 
;;;   
;;;   If an item exists in the filter list and you elect to add a new item, 
;;;   the old value is overwritten by the new value, as you can have only 
;;;   one of each type in a single (ssget "x") call.
;;;
;;;--------------------------------------------------------------------------;
;;;
;;; Find the dotted pairs that are valid filters for ssget
;;; in entity named "ent".
;;;
;;; ssx_fe == SSX_Find_Entity
;;;
(defun ssx_fe (/ x data fltr ent)
  (setq ent (car (entsel "\nSelect object/<None>: ")))
  (if ent
    (progn
      (setq data (entget ent))
      (foreach x '(0 2 8 6 62 7 40 50 39 210) ; do not include 38
        (if (assoc x data)
          (setq fltr 
            (cons (assoc x data) fltr)
          )
        )
      )    
      (reverse fltr)
    )
  ) 
)
;;;
;;; Remove "element" from "alist".
;;;
;;; ssx_re == SSX_Remove_Element
;;;
(defun ssx_re (element alist)
  (append
    (reverse (cdr (member element (reverse alist))))
    (cdr (member element alist))   
  )
)
;;;
;;; INTERNAL ERROR HANDLER 
;;;
;(defun ssx_er (s)                     ; If an error (such as CTRL-C) occurs
;                                      ; while this command is active...
;  (if (/= s "Function cancelled")
;    (princ (strcat "\nError: " s))
;  )
;  (if olderr (setq *error* olderr))   ; Restore old *error* handler
;  (princ)
;)
;;;   
;;; Get the filtered sel-set.
;;;
;;;
(defun ssx (/ olderr)
  (gc)                                ; close any sel-sets            
;  (setq olderr *error* 
;        *error* ssx_er 
;  )
;  (setq fltr (ssx_fe)) ;remove initial get
  (ssx_gf fltr) 
)
;;;
;;; Build the filter list up by picking, selecting an item to add,
;;; or remove an item from the list by selecting it and pressing RETURN.
;;;
;;; ssx_gf == SSX_Get_Filters
;;;
(defun ssx_gf (f1 / t1 t2 t3 f2 pcatch)
(setq pcatch 1)
(if v_qcatch (setq pcatch nil v_qcatch nil))
(if v_ssxcatch (setq t1 nil f1 nil t2 "CAtch" t3 v_ssxcatch v_ssxcatch nil) (setq t1 ""))
  (while t1
    (progn
      (cond (f1 (prompt "\nFilter: ") (prin1 f1)))
      (initget 
        (strcat "Block Color Entity LAyer LType Text STyle SCale Ro THickness Ucs "
          "0 2 8 6 62 3 7 40 39 50 210 All CAtch Pick X"))
           
      (setq t1 (getkword (strcat
"\n   0     2     8     6     62    1   3,7   40     50      39..CAtch/All/eXit"
"\nEntity/Block/LAyer/LType/Color/Text/STyle/SCale/Rotation/THickness/Ucs/"
        (if f1 "Pick: " "<Pick>:")))) 
    )
    (setq t2
      (cond
        ((not t1)(if f1 nil (setq t1 "Pick")))
        ((eq t1 "Block")      2)   ((eq t1 "Color")      62)
        ((eq t1 "Entity")     0)   ((eq t1 "LAyer")       8)
        ((eq t1 "LType")      6)   ((eq t1 "SCale")      40)
        ((eq t1 "STyle")      7)   ((eq t1 "THickness")  39)
        ((eq t1 "Flag" )     66)   ((eq t1 "Ucs")       210)
        ((eq t1 "Pick" ) "Pick")   ((eq t1 "CAtch") "CAtch")
        ((eq T1 "All")    "All")   ((eq t1 "Text")        1)
        ((eq t1 "Ro")        50)   ((eq t1 "X")          "X")
        (T (if t1(read t1)""))
      )
    )
    (setq t3
      (cond
        ((= t2  0) (getstring "\n>>Entity type to Find/<RETURN to remove>: "))
        ((= t2  1)  (getstring t "\n>>Wildcard Text value to Find/<RETURN to remove>: "))
        ((= t2  2)  (getstring "\n>>Block name to Find/<RETURN to remove>: "))
        ((= t2  6) (getstring "\n>>Linetype name to Find/<RETURN to remove>: "))
        ((= t2  7) 
          (getstring "\n>>Text/Dim style name to Find/<RETURN to remove>: "))
        ((= t2  8) (getstring "\n>>Layer name to Find/<RETURN to remove>: "))
        ((= t2 39)  (getreal   "\n>>Thickness to Find/<RETURN to remove>: "))
        ((= t2 40)  (getreal   "\n>>Object Scale to Find/<RETURN to remove>: "))
        ((= t2 50)  (if(setq ro(getreal   "\n>>Degrees of Rotation to Find /<RETURN to remove>: "))
                       (* pi (/ ro 180)) nil))
        ((= t2 62)  (initget 4 "?")
          (cond
            ((or (eq (setq t3 (getint 
              "\n>>Color number to Find/?/<RETURN to remove>: ")) "?") 
              (> t3 256))
              (ssx_pc)                ; Print color values.
              nil
            )
            (T
              t3                      ; Return t3.
            )
          )
        )
        ((= t2 210) (getpoint  "\n>>UCS Extrusion Vector to Find/<RETURN to remove>: ")
        )
        (T          nil)
      )
    )
    (cond
      ((= t2 "Pick") (setq f1 (ssx_fe) t2 nil)) ; get entity
      ((= t2 "CAtch") (setq t1 nil f1 nil t3 1)) ; get last created entities
      ((= t2 "All") (setq t1 nil f1 nil t3 1)) ; get all entities
      ((= t2 "X") (setq t1 nil f1 nil t3 1)) ; exit
      ((and f1 (assoc t2 f1))         ; already in the list
        (if (and t3 (/= t3 ""))
          ;; Replace with a new value...             
          (setq f1 (subst (cons t2 t3) (assoc t2 f1) f1)) 
          ;; Remove it from filter list...
          (setq f1 (ssx_re (assoc t2 f1) f1)) 
        )  
      )
      ((and t3 (/= t3 ""))
        (cond
        ((and (= t2 1)(setq typ (dxf 0 f1))
              (not (member (strcase typ) '( "TEXT" "ATTDEF" "DIMENSION" "ATTRIB"))))
            (setq f1 (cons (cons 0 "Text") f1)))
        ((= t2 7) 
            (initget 1 "D T")
            (if (= "T" (getstring "\nDimension or Text Style ? {Dim Text}: "))
                  (setq f1 (cons (cons 7 t3) f1))
                  (setq f1 (cons (cons 3 t3) f1))
                  )
           )
        (T (setq f1 (cons (cons t2 t3) f1)))
        )
        )
      (T nil)
    )
  )
  (cond (f1 (prompt "\nMake Selection to Filter<global>")
            (if (ssget) (setq f2 (ssget "p" f1))(setq f2 (ssget "x" f1))))
        ((= t2 "CAtch") (setq f1 T2 f2 (f_catch t3 pcatch)))
        ((= t2 "All") (setq f1 T2 f2 (SSGET "X")))
        ((= t2 "X") (setq f1 nil))
        (t nil))
  (if (and f1 f2) 
    (progn
      (princ (strcat "\n" (itoa (sslength f2)) " found."))
      f2 
    )
    (progn (princ "\n0 found.") nil)
  )
)
;;;
;;; Print the standard color assignments.
;;;
;;;
(defun ssx_pc ()
  (if textpage (textpage) (textscr))
  (princ "\n                                                     ")
  (princ "\n                 Color number   |   Standard meaning ")
  (princ "\n                ________________|____________________")
  (princ "\n                                |                    ")
  (princ "\n                       0        |      <BYBLOCK>     ")
  (princ "\n                       1        |      Red           ")
  (princ "\n                       2        |      Yellow        ")
  (princ "\n                       3        |      Green         ")
  (princ "\n                       4        |      Cyan          ")
  (princ "\n                       5        |      Blue          ")
  (princ "\n                       6        |      Magenta       ")
  (princ "\n                       7        |      White         ")
  (princ "\n                    8...255     |      -Varies-      ")
  (princ "\n                      256       |      <BYLAYER>     ")
  (princ "\n                                               \n\n\n")
)
;;;
;;; C: function definition.
;;;
(defun c:ssx () (ssx)(princ))
(princ)
